#!/usr/bin/env python3
"""
Network Bandwidth Tester
Parameters:
    - IP: Target IP address
    - port: Target port number
"""
import socket
import time
import threading
import argparse
from statistics import mean

class BandwidthTest:
    def __init__(self, ip, port):
        self.ip = ip
        self.port = port
        self.server_running = False
        self.results = []
        
    def start_server(self):
        server = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        server.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
        server.bind((self.ip, self.port))
        server.listen(1)
        self.server_running = True
        
        while self.server_running:
            client, addr = server.accept()
            data = client.recv(1024*1024)  # 1MB buffer
            client.send(b'ACK')
            client.close()
            
    def run_client(self):
        # Create test data (1MB)
        data = b'x' * (1024 * 1024)
        
        for _ in range(5):  # 5 test runs
            try:
                start_time = time.time()
                
                # Connect and send data
                sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
                sock.connect((self.ip, self.port))
                sock.send(data)
                
                # Wait for acknowledgment
                sock.recv(3)
                sock.close()
                
                # Calculate speed
                end_time = time.time()
                duration = end_time - start_time
                speed = (len(data) / duration) / (1024 * 1024)  # MB/s
                self.results.append(speed)
                
                print(f"Test {len(self.results)}: {speed:.2f} MB/s")
                time.sleep(1)
                
            except Exception as e:
                print(f"Error during test: {str(e)}")
                
        if self.results:
            print(f"\nAverage Speed: {mean(self.results):.2f} MB/s")
            print(f"Max Speed: {max(self.results):.2f} MB/s")
            print(f"Min Speed: {min(self.results):.2f} MB/s")

def run_test(ip, port, mode='client'):
    tester = BandwidthTest(ip, port)
    
    if mode == 'server':
        print(f"Starting bandwidth test server on {ip}:{port}")
        tester.start_server()
    else:
        print(f"Starting bandwidth test client, connecting to {ip}:{port}")
        tester.run_client()

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Network Bandwidth Tester")
    parser.add_argument("ip", help="Target IP address")
    parser.add_argument("port", type=int, help="Target port number")
    parser.add_argument("--mode", choices=['client', 'server'],
                      default='client', help="Operation mode")
    args = parser.parse_args()
    
    run_test(args.ip, args.port, args.mode)
